@@ -15,7 +15,7 @@
 #include <type_traits>
 #include <vector>
 
-#include "gtest/gtest.h"
+#include "doctest/doctest.h"
 
 #include "xsimd/memory/xsimd_aligned_allocator.hpp"
 #include "xsimd/memory/xsimd_alignment.hpp"
@@ -24,7 +24,7 @@ struct mock_container
 {
 };
 
-TEST(xsimd, alignment)
+TEST_CASE("[alignment]")
 {
     using u_vector_type = std::vector<double>;
     using a_vector_type = std::vector<double, xsimd::default_allocator<double>>;
@@ -33,8 +33,8 @@ TEST(xsimd, alignment)
     using a_vector_align = xsimd::container_alignment_t<a_vector_type>;
     using mock_align = xsimd::container_alignment_t<mock_container>;
 
-    EXPECT_TRUE((std::is_same<u_vector_align, xsimd::unaligned_mode>::value));
-    EXPECT_TRUE((std::is_same<a_vector_align, xsimd::aligned_mode>::value));
-    EXPECT_TRUE((std::is_same<mock_align, xsimd::unaligned_mode>::value));
+    CHECK_UNARY((std::is_same<u_vector_align, xsimd::unaligned_mode>::value));
+    CHECK_UNARY((std::is_same<a_vector_align, xsimd::aligned_mode>::value));
+    CHECK_UNARY((std::is_same<mock_align, xsimd::unaligned_mode>::value));
 }
 #endif
